const path = require('path')
const fs = require('fs')
const utils = require('./utils')
const webpack = require('webpack')
const config = require('../config')
const merge = require('webpack-merge')
const baseWebpackConfig = require('./webpack.base.conf')
const CopyWebpackPlugin = require('copy-webpack-plugin')

const env = require('../config/prod.env')
function resolve (dir) {
  return path.join(__dirname, '..', dir)
}

var externals = {};
const utilsList = fs.readdirSync(path.resolve(__dirname, '../src/utils'));
utilsList.forEach(file => {
  var file = path.basename(file, '.js');
  externals[`cvtsp-components/src/utils/${file}`] = `cvtsp-components/lib/utils/${file}`;
})
externals = Object.assign({
  vue: {
    root: 'Vue',
    commonjs: 'vue',
    commonjs2: 'vue',
    amd: 'vue'
  }
}, externals);

const webpackConfig = merge({}, {
    output: {
      path: path.resolve(__dirname, '../lib'),
      filename: '[name].js'
    },
    externals: externals,
    resolve: {
      extensions: ['.js', '.vue', '.json'],
      alias: {
        '@': resolve('src'),
        'components': resolve('src/components'),
        'utils': resolve('src/utils'),
        'mixins': resolve('src/mixins') 
      }
    },
    plugins: [
      // http://vuejs.github.io/vue-loader/en/workflow/production.html
      new webpack.DefinePlugin({
        'process.env': env
      }),
      new webpack.HashedModuleIdsPlugin(),
      // enable scope hoisting
      new webpack.optimize.ModuleConcatenationPlugin(),
      new CopyWebpackPlugin([
        {
          from: path.resolve(__dirname, '../src/utils'),
          to: 'utils',//config.build.assetsSubDirectory,
          //ignore: ['.*']
        }
      ])
    ]
})


webpackConfig.entry = utils.getEntries('src/utils/**/*.js');

module.exports = webpackConfig;