import '../style/index.scss';

export default {
    name: 'CvTabsSlots',
    props: {
        titles: [Array, Function],
        titlesLeft: Function,
        titlesRight: Function,
        // 默认只渲染当前tab页
        forceRender: {
            type: [Boolean],
            default: true
        },
        visible: {
            type: Boolean,
            default: false
        },
        value: {
            default: 0
        }
    },
    data: function data() {
        return {
            active: 0,
            tabsCache: {}
        };
    },

    computed: {
        content_show: {
            get: function get() {
                return this.visible;
            }
        }
    },
    created: function created() {
        this.tabsCache = this.detailCache(this.active, this.forceRender);
    },

    watch: {
        visible: {
            immediate: true,
            handler: function handler(val) {
                var content = this.$refs['content'];
                if (content) {
                    val ? content.style.display = 'block' : content.style.display = 'none';
                }
            }
        }
    },
    render: function render(h) {
        var _this = this;

        var titles = this.titles.call(this, h).map(function (title, index) {
            return h(
                'span',
                { 'class': ['titles-middle-item', _this.active == index ? 'middle-item-active' : ''],
                    on: {
                        'click': _this.handlerTabs.bind(_this, index)
                    }
                },
                [title]
            );
        });
        var childs = this.$slots.default && this.$slots.default.filter(function (child) {
            return child.tag;
        }).map(function (child, index) {
            return h(
                'div',
                { 'class': ['bottom-tabs-panel', _this.active == index ? 'bottom-tabspanel-active' : ''] },
                [_this.tabsCache[index] && child]
            );
        });
        return h(
            'div',
            { 'class': 'cv-bottom-tabs' },
            [h(
                'ul',
                { 'class': 'bottom-tabs-header' },
                [h(
                    'li',
                    { 'class': 'header-titles header-titles-left' },
                    [this.titlesLeft && this.titlesLeft.call(this, h)]
                ), h(
                    'li',
                    { 'class': 'header-titles header-titles-middle' },
                    [titles]
                ), h(
                    'li',
                    { 'class': 'header-titles header-titles-right' },
                    [this.titlesLeft && this.titlesRight.call(this, h)]
                )]
            ), h(
                'div',
                { 'class': 'bottom-tabs-content', ref: 'content' },
                [childs]
            )]
        );
    },

    methods: {
        handlerTabs: function handlerTabs(index) {
            this.active = index;
            this.tabsCache[index] = true;

            this.$emit('input', index);
            this.$emit('update:visible', true);
            this.$emit('tabs-click', index);
        },


        /**
         * 得到contentdom对象
         */
        getContent: function getContent() {
            return this.$refs['content'];
        },


        /**
         * 将当前的activeName作为缓存(tabsCache)
         * @param {string} activeName: 点击的当前活动页键值
         * @param {Boolean} forceRender: 是否缓存所有tab(true只缓存当前tab页)
         * @return {Object} cacheLists: {tabs_0: true, tabs_1: false}
         */
        detailCache: function detailCache(activeName, forceRender) {
            var cacheLists = {};
            var titles = Array.isArray(this.titles) ? this.titles : this.titles.call(this);

            titles.forEach(function (title, index) {
                if (!forceRender) {
                    cacheLists[index] = true;
                } else {
                    if (activeName == index) {
                        cacheLists[index] = true;
                    } else {
                        cacheLists[index] = false;
                    }
                }
            });
            return cacheLists;
        }
    }
};