import { on } from 'cvtsp-components/lib/utils/element';

var drname = 'cv-drag-boundary-';
var initialHeight = 200;
var limitDistance = 400;
function createDragElement(el, binding, vnode) {
    var div = document.createElement('div');
    div.className = drname + el[drname].position || 'top';
    setStyle(div, {
        position: 'absolute',
        top: 0,
        left: 0,
        width: '100%',
        height: '3px',
        cursor: 'ns-resize'
    });
    el.appendChild(div);

    var dragItem = el.querySelector('.' + drname + el[drname].position);
    createDragHandler(dragItem, el, binding);
}

function setStyle(ele) {
    var styles = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    for (var i in styles) {
        ele.style[i] = styles[i];
    }
}

function createDragHandler(el, parent, binding) {
    // 需要变化高度的box
    var _changeBox = parent[drname].target;
    if (!_changeBox) return;

    _changeBox.style.height = initialHeight + 'px';
    if (el && el.nodeType == 1) {
        // drag移动事件
        var mousemove = function mousemove(e) {
            var initial = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
            var livingHeight = arguments[2];

            var event = e || window.event;
            var currentHeight = initial - event.clientY;
            event.preventDefault();

            if (livingHeight + currentHeight > limitDistance) {
                _changeBox.style.height = limitDistance + 'px';
            } else {
                _changeBox.style.height = livingHeight + currentHeight + 'px';
            }
        };
        // drag离开事件
        var moveup = function moveup(e) {
            document.onmousemove = null;
            document.onmouseup = null;
        };

        on(el, 'mousedown', function (e) {
            var event = e || window.event;
            var height = parseInt(_changeBox.style.height) || initialHeight;

            event.preventDefault();
            document.onmousemove = function (e) {
                mousemove(e, event.clientY, height);
            };
            document.onmouseup = function (e) {
                moveup(e);
            };
        });
    } else {
        throw new Error('drag element is not exitst');
    }
}

export default {
    bind: function bind(el, binding, vnode) {
        var args = binding.arg ? binding.arg.split(':') : [];
        var dragbox = vnode.context.$refs[args[0]] ? vnode.context.$refs[args[0]] : vnode.context.$children[0].$refs[args[0]];
        el[drname] = {
            target: dragbox,
            position: args[1] || 'top'
        };
        createDragElement(el, binding, vnode);
    }
};