import _Object$assign from 'babel-runtime/core-js/object/assign';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';

var TorqueCanvas = function () {
    function TorqueCanvas(options) {
        _classCallCheck(this, TorqueCanvas);

        this.options = _Object$assign({
            padding: 30
        }, options);
        this.canvas = this._createCanvas(this.options.el);

        this.canvas.translate(this.options.padding, this.options.padding);

        this.getDataRenderLine(this.options.vertical, this.options.horizontal).renderCoordinate();

        // 测试一个矩形
        this._drawRectInWord({ x: 100, y: 100 }, 100, 40, 'hello');
        this._drawRectInWord({ x: 200, y: 200 }, 100, 40, 'world', { fillStyle: 'red' });
        // 测试一个线性
        this._drawLine({ x: 20, y: 30 }, { x: 100, y: 300 }, 'line', { strokeStyle: 'blue', lineWidth: 6 });
    }
    /**
     * 创建canvas画布对象
     * @param {*} el 
     * @return canvas
     */


    _createClass(TorqueCanvas, [{
        key: '_createCanvas',
        value: function _createCanvas(el) {
            if (el && el.nodeType == 1) {
                return el.getContext('2d');
            }
        }

        /**
         * 重新渲染数据
         * @param {Object} options={horizontal=[], vertical=[]} 
         */

    }, {
        key: 'initialRender',
        value: function initialRender() {
            var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

            this.getDataRenderLine(options.vertical, options.horizontal);
        }

        /**
         * 根据数据渲染虚线
         * @param {Array} vertical:垂直坐标(从大到小)
         * @param {Array} horizontal: 水平坐标 
         */

    }, {
        key: 'getDataRenderLine',
        value: function getDataRenderLine() {
            var _this = this;

            var vertical = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
            var horizontal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

            var vertical = this.units_arraySortDown(vertical);
            var horizontal = this.units_arraySortUp(horizontal);

            var _getDataCalcCellSize = this.getDataCalcCellSize(vertical, horizontal),
                verticalSize = _getDataCalcCellSize.verticalSize,
                horizontalSize = _getDataCalcCellSize.horizontalSize,
                width = _getDataCalcCellSize.width,
                height = _getDataCalcCellSize.height;

            // 垂直


            vertical.forEach(function (v, index) {
                var start = { x: 0, y: height * index };
                var end = { x: horizontalSize, y: height * index };
                _this._drawLine(start, end);
                _this._drawText(v, start.x - 15, start.y + 5);
            });

            horizontal.forEach(function (h, index) {
                var start = { x: width * index, y: 0 };
                var end = { x: width * index, y: verticalSize };
                _this._drawLine(start, end);
                _this._drawText(h, start.x, verticalSize + 5);
            });

            return this;
        }

        /**
         * 根据数据计算单位长和宽
         * @param {Array} vertial: 垂直的数据
         * @param {Array} horizontal: 水平的数据
         * @return {Object}
         */

    }, {
        key: 'getDataCalcCellSize',
        value: function getDataCalcCellSize() {
            var vertical = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
            var horizontal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

            var width = Math.ceil((this.options.el.width - this.options.padding * 2) / horizontal.length),
                height = Math.ceil((this.options.el.height - this.options.padding * 2) / vertical.length);

            return {
                width: width,
                height: height,
                horizontalSize: horizontal.length * width,
                verticalSize: vertical.length * height
            };
        }

        /**
         * 初始化坐标系--one step
         */

    }, {
        key: 'renderCoordinate',
        value: function renderCoordinate() {
            var coordinate_xdistance = this.options.el.width - this.options.padding * 2;
            var coordinate_ydistance = this.options.el.height - this.options.padding * 2;

            // y轴
            this._drawLine({ x: 0, y: 0 }, { x: 0, y: coordinate_ydistance }, 'line', {
                strokeStyle: 'red'
            });
            // x轴
            this._drawLine({ x: 0, y: coordinate_ydistance }, { x: coordinate_xdistance, y: coordinate_ydistance }, 'line', {
                strokeStyle: 'red'
            });
            return this;
        }

        /**
         * 画线
         * @param {Object} start={x, y} 开始的点坐标
         * @param {Object} end={x, y} 结束的点坐标
         * @param {String} type: 虚线和实线
         * @param {Object} options: 配置项
         */

    }, {
        key: '_drawLine',
        value: function _drawLine(start, end) {
            var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'dash';
            var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

            this.canvas.lineWidth = options.lineWidth || 1;
            this.canvas.strokeStyle = options.strokeStyle || '#303431';

            this.canvas.beginPath();
            this.canvas.moveTo(start.x, start.y);
            this.canvas.lineTo(end.x, end.y);
            //type == 'dash' ? this.canvas.setLineDash([10, 3]) : null;
            this.canvas.stroke();
            this.canvas.closePath();
        }

        /**
         * 绘制文本
         * @param {String} text 
         * @param {*} x 
         * @param {*} y 
         */

    }, {
        key: '_drawText',
        value: function _drawText(text, x, y) {
            this.canvas.fillStyle = 'black';
            this.canvas.textAlign = 'center';
            this.canvas.textBaseline = 'middle';
            this.canvas.font = '24px serif';
            this.canvas.fillText(text, x, y);
        }

        /**
         * 绘制带有文字的矩形
         * @param {Object} start={x, y} 开始坐标 
         * @param {*} width 
         * @param {*} height 
         * @param {Object} options:配置项
         */

    }, {
        key: '_drawRectInWord',
        value: function _drawRectInWord(start, width, height, text) {
            var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
            var x = start.x,
                y = start.y;


            this.canvas.fillStyle = options.fillStyle || '#ccc';
            this.canvas.fillRect(x, y, width, height);

            this._drawText(text, x + width / 2, y + height / 2);
        }

        /**
         * 数组从小到大排序
         * @param {Array} array 
         */

    }, {
        key: 'units_arraySortUp',
        value: function units_arraySortUp(array) {
            return array.sort(function (a, b) {
                return a - b;
            });
        }

        /**
         * 数组从大到小排序
         * @param {Array} array 
         */

    }, {
        key: 'units_arraySortDown',
        value: function units_arraySortDown(array) {
            return array.sort(function (a, b) {
                return b - a;
            });
        }
    }]);

    return TorqueCanvas;
}();

export default TorqueCanvas;