import '../style/index.scss';

export default {
    name: 'CvTabsSlots',
    props: {
        titles: [Array, Function],
        titlesLeft: Function,
        titlesRight: Function,
        // 默认只渲染当前tab页
        forceRender: {
            type: [Boolean],
            default: true
        },
        visible: {
            type: Boolean,
            default: false
        },
        value: {
            default: 0
        }
    },
    data() {
        return {
            active: 0,
            tabsCache: {}
        }
    },
    computed: {
        content_show: {
            get() {
                return this.visible;
            }
        }
    },
    created() {
        this.tabsCache = this.detailCache(this.active, this.forceRender);
    },
    watch: {
        visible: {
            immediate: true,
            handler(val) {
                const content = this.$refs['content'];
                if(content) {
                    val ? content.style.display = 'block' : content.style.display = 'none';
                }
            }
        }
    },
    render(h) {
        const titles = this.titles.call(this, h).map((title, index) => {
            return (
                <span class={['titles-middle-item', this.active == index ? 'middle-item-active' : '']} 
                onClick={this.handlerTabs.bind(this, index)}>{title}</span>
            )
        });
        const childs = 
        this.$slots.default 
        && this.$slots.default.filter(child => child.tag).map((child, index) => {
            return (
                <div class={['bottom-tabs-panel', this.active == index ? 'bottom-tabspanel-active' : '']}>
                    {this.tabsCache[index] && child}
                </div>
            )
        })
        return (
            <div class="cv-bottom-tabs">
                <ul class="bottom-tabs-header">
                    <li class="header-titles header-titles-left">
                        {this.titlesLeft && this.titlesLeft.call(this,h)}
                    </li>
                    <li class="header-titles header-titles-middle">{titles}</li>
                    <li class="header-titles header-titles-right">
                        {this.titlesLeft && this.titlesRight.call(this,h)}
                    </li>
                </ul>
                <div class="bottom-tabs-content" ref="content">
                    {childs}
                </div>
            </div>
        )
    },
    methods: {
        handlerTabs(index) {
            this.active = index;
            this.tabsCache[index] = true;
            
            this.$emit('input', index);
            this.$emit('update:visible', true);
            this.$emit('tabs-click', index);
        },

        /**
         * 得到contentdom对象
         */
        getContent() {
            return this.$refs['content'];
        },

        /**
         * 将当前的activeName作为缓存(tabsCache)
         * @param {string} activeName: 点击的当前活动页键值
         * @param {Boolean} forceRender: 是否缓存所有tab(true只缓存当前tab页)
         * @return {Object} cacheLists: {tabs_0: true, tabs_1: false}
         */
        detailCache(activeName, forceRender) {
            var cacheLists = {};
            var titles = Array.isArray(this.titles) ? this.titles : this.titles.call(this);

            titles.forEach( (title, index) => {
                if(!forceRender) {
                    cacheLists[index] = true;
                }else {
                    if(activeName == index) {
                        cacheLists[index] = true;
                    }else {
                        cacheLists[index] = false;
                    }   
                }
            });
            return cacheLists;
        }
    }
}