import {on} from 'cvtsp-components/src/utils/element'

const drname = 'cv-drag-boundary-';
const initialHeight = 200;
const limitDistance = 400;
function createDragElement(el, binding, vnode) {
    const div = document.createElement('div');
    div.className = drname + el[drname].position || 'top';
    setStyle(div, {
        position: 'absolute',
        top: 0,
        left: 0,
        width: '100%',
        height: '3px',
        cursor: 'ns-resize'
    });
    el.appendChild(div);

    const dragItem = el.querySelector('.' + drname + el[drname].position);
    createDragHandler(dragItem, el, binding);
}

function setStyle(ele, styles={}) {
    for(var i in styles) {
        ele.style[i] = styles[i];
    }
}

function createDragHandler(el, parent, binding) {
    // 需要变化高度的box
    const _changeBox = parent[drname].target;
    if(!_changeBox) return;

    _changeBox.style.height = initialHeight + 'px';
    if(el && el.nodeType == 1) {
        // drag移动事件
        const mousemove = (e, initial=0, livingHeight) => {
            const event = e || window.event;
            const currentHeight = initial - event.clientY;
            event.preventDefault();

            if((livingHeight + currentHeight) > limitDistance) {
                _changeBox.style.height = limitDistance + 'px';
            }else {
                _changeBox.style.height = (livingHeight + currentHeight) + 'px';
            }
        };
        // drag离开事件
        const moveup = e => {
			document.onmousemove = null;
			document.onmouseup = null;
        };
        
        on(el, 'mousedown', e => {
            const event = e || window.event;
            const height = parseInt(_changeBox.style.height) || initialHeight;

            event.preventDefault();
            document.onmousemove = (e) => {
                mousemove(e, event.clientY, height);			
            };
            document.onmouseup = (e) => {
                moveup(e);
            }
        })
    }else {
        throw new Error('drag element is not exitst');
    }
}

export default {
    bind(el, binding, vnode) {
        const args = binding.arg ? (binding.arg).split(':') : [];
        const dragbox = vnode.context.$refs[args[0]] ? vnode.context.$refs[args[0]] : vnode.context.$children[0].$refs[args[0]]
        el[drname] = {
            target: dragbox,
            position: args[1] || 'top'
        };
        createDragElement(el, binding, vnode);   
    }
}