/*
* name: 所有dom操作的简单方法
* author: wukangjun
* time: 2017.07.10
*/
var slice = Array.prototype.slice;
var myJq = {};

export const trim = function(string) {
  return (string || '').replace(/^[\s\uFEFF]+|[\s\uFEFF]+$/g, '');
}

// 绑定事件 -- on
export const on = function(element, event, handler){
  if (document.addEventListener) {
    return function(element, event, handler) {
      if (element && event && handler) {
        element.addEventListener(event, handler, false);
      }
    };
  } else {
    return function(element, event, handler) {
      if (element && event && handler) {
        element.attachEvent('on' + event, handler);
      }
    };
  }
}()

// 解除绑定 --off
export const off = function(element, event, handler){
  if (document.removeEventListener) {
    return function(element, event, handler) {
      if (element && event) {
        element.removeEventListener(event, handler, false);
      }
    };
  } else {
    return function(element, event, handler) {
      if (element && event) {
        element.detachEvent('on' + event, handler);
      }
    };
  }
}()

// 设置样式
export const css = function(el, name, config){
  if(el){
    if(arguments.length > 2){
      el.style[name] = config;
    }else{
      return el.style[name]
    }
  }
} 

/**
 * 委托元素
 * @param {*} event 
 * @param {Element | RegExp} regexp: class的样式名  element: 元素 
 */
const elementList = ['html', 'div', 'a', 'ul', 'li', 'em', 'span'];
export const delegation = (event, regexp) => {
  let target = event.target, flag = false;
  // element
  if(elementList.indexOf(regexp) >= 0) {
    flag = true;
  }

  while(target && target.tagName.toUpperCase() !== 'HTML'){
    if(flag && target.tagName.toLowerCase() === regexp) {
      return target;
    }else {
      const regs = target.className.split(/\s+/g);
      for(let i = 0, len=regs.length; i<len; i++) {
          if(new RegExp('^'+regexp+'$').test(regs[i])) {
              return target;
          }
      }
    }
    target = target.parentNode;
  }
  return null;
}

/**
 * 
 * @param {*Element} element: 父元素
 * @param {* String} eventName: 事件名称
 * @param {* String} delegator: 子元素类名
 * @param {* Function} handler: 方法 
 */
export const handler = function(element, eventName, delegator, handler){
  element.addEventListener(eventName, function(e){
    const _this = deletator(e, delegator);
    _this && handler.apply(_this, arguments);
  }, false);

  function deletator(e, name){
    let target = (e || windowevent).target;
    let reg = new RegExp(name, 'g');
    while(target && target.tagName.toUpperCase() !== 'HTML'){
      if(reg.test(target.className)) {
        return target;
      }
      target = target.parentNode;
    }
    return null;
  }
}
